@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2020 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -9,9 +9,7 @@
 #ifndef SQUID_SRC_TEST_TESTSBUF_H
 #define SQUID_SRC_TEST_TESTSBUF_H
 
-#include <cppunit/extensions/HelperMacros.h>
-
-#include "OutOfBoundsException.h"
+#include "compat/cppunit.h"
 
 class testSBufList : public CPPUNIT_NS::TestFixture
 {
